@@ -11,7 +11,7 @@ Pandoc - interface to the Pandoc document converter
 
 =cut
 
-our $VERSION = '0.2.0';
+use version 0.77; our $VERSION = version->declare('0.2.1');
 
 use Carp 'croak';
 use IPC::Run3;
@@ -31,16 +31,20 @@ sub import {
 }
 
 sub new {
-    my $out;
-    my $in;
+    my ($class, %opts) = @_;
+    my ($in, $out);
 
-    run3 ['pandoc','-v'], \$in, \$out, \undef,
+    my $pandoc = bless { 
+        bin => $opts{bin} // 'pandoc'
+    }, $class;
+    
+    run3 [ $pandoc->{bin},'-v'], \$in, \$out, \undef,
         { return_if_system_error => 1 };
 
     croak "pandoc executable not found\n" unless
         $out and $out =~ /^pandoc (\d+(\.\d+)+)/;
 
-    my $pandoc = bless { version => $1 }, 'Pandoc';
+    $pandoc->{version} = version->parse($1);
     $pandoc->{data_dir} = $1 if $out =~ /^Default user data directory: (.+)$/m;
 
     return $pandoc;
@@ -48,7 +52,7 @@ sub new {
 
 sub pandoc(@) { ## no critic
     $PANDOC //= eval { Pandoc->new } // 0;
-    
+
     if (@_) {
         return $PANDOC ? $PANDOC->run(@_) : -1;
     } else {
@@ -60,7 +64,7 @@ sub run {
     my $pandoc = shift;
 
     # We shift/pop these args but want to remember what reftype they were
-    my %is_ref = ( args => ('ARRAY' eq ref $_[0] ), 
+    my %is_ref = ( args => ('ARRAY' eq ref $_[0] ),
                    opts => ('HASH' eq ref $_[-1]) );
     my @args   = $is_ref{args} ? @{ shift @_ } : ();
     my %opts   = $is_ref{opts} ? %{pop @_} : ();
@@ -86,7 +90,7 @@ sub run {
     my $err = $opts{err};
     $opts{return_if_system_error} //= 1;
 
-    for my $io ( qw[ in out err ] ) {
+    for my $io ( qw(in out err) ) {
         $opts{"binmode_std$io"} //= $opts{binmode} if $opts{binmode};
         if ( 'SCALAR' eq ref $opts{$io} ) {
             next unless utf8::is_utf8(${$opts{$io}});
@@ -94,27 +98,24 @@ sub run {
         }
     }
 
-    run3 ['pandoc', @args ], $in, $out, $err, \%opts;
+    run3 [ $pandoc->{bin}, @args ], $in, $out, $err, \%opts;
 
     return $? == -1 ? -1 : $? >> 8;
 }
 
 sub convert {
     my $pandoc = shift;
-    $pandoc = do { $PANDOC //= Pandoc->new } if $pandoc eq 'Pandoc'; 
-    return unless $pandoc;
-
-    my $from  = shift;
-    my $to    = shift;
-    my $in    = shift;
-    my $out   = "";
-    my $err   = "";
+    my $from   = shift;
+    my $to     = shift;
+    my $in     = shift;
+    my $out    = "";
+    my $err    = "";
 
     my $utf8 = utf8::is_utf8($in);
 
-    my $status = $pandoc->run( [ '-f' => $from, '-t' => $to, @_ ], 
+    my $status = $pandoc->run( [ '-f' => $from, '-t' => $to, @_ ],
         in => \$in, out => \$out, err => \$err );
-    
+
     croak($err || "pandoc failed with exit code $status") if $status;
 
     utf8::decode($out) if $utf8;
@@ -124,42 +125,57 @@ sub convert {
 }
 
 sub require {
-    my $pandoc = @_ < 2 
-        ? $PANDOC //= Pandoc->new   # may throw
-        : do { ref $_[0] ? shift : shift->new };
-    my $version = shift;
-
+    my ($pandoc, $version) = @_;
+    $pandoc = do { $PANDOC //= Pandoc->new } if $pandoc eq 'Pandoc'; 
     croak "pandoc $version required, only found ".$pandoc->{version}."\n"
         unless $pandoc->version($version);
+    return $pandoc;
 }
 
 sub version {
     my $pandoc = shift;
-    $pandoc = do { $PANDOC //= Pandoc->new } if $pandoc eq 'Pandoc'; 
     return unless $pandoc and $pandoc->{version};
 
-    if (@_) { # compare against given version
-        my $version = shift;
-        croak "invalid version number: $version\n"
-            if $version !~ $PANDOC_VERSION_SPEC;
-
-        my @got = split /\./, $pandoc->{version};
-        foreach my $e (split /\./, $version) {
-            my $g = shift @got // 0;
-            return if $e > $g;
-            last   if $e < $g;
-        }
+    # compare against given version
+    if (@_) {
+        my $version = eval { version->parse($_[0]) } or
+            croak "Invalid version format: $_[0]";
+        return if $version > $pandoc->{version};
     }
 
     return $pandoc->{version};
 }
 
 sub data_dir {
-    my $pandoc = shift;
-    $pandoc = do { $PANDOC //= Pandoc->new } if $pandoc eq 'Pandoc'; 
-    return unless $pandoc;
+    $_[0]->{data_dir};
+}
 
-    $pandoc->{data_dir};
+sub _help { # not documented. may change to return structured data
+    my ($pandoc) = @_;
+
+	unless (defined $pandoc->{help}) {
+		my $help;
+    	$pandoc->run('--help', { out => \$help });
+		for my $inout (qw(Input Output)) {
+			$help =~ /^$inout formats:\s+([a-z_0-9,\+\s*]+)/m;
+			$pandoc->{lc($inout).'_formats'} = [ split /,\s+|\s+/, $1 ]
+		}
+	    $pandoc->{help} = $help;
+    }
+
+	$pandoc->{help} 
+}
+
+sub input_formats {
+    my ($pandoc) = @_;
+	$pandoc->_help;
+    @{$pandoc->{input_formats}};
+}
+
+sub output_formats {
+    my ($pandoc) = @_;
+	$pandoc->_help;
+    @{$pandoc->{output_formats}};
 }
 
 1;
@@ -182,7 +198,7 @@ __END__
 
   use Pandoc;             # check at first use
   use Pandoc 1.12;        # check at compile time
-  pandoc->require(1.12);  # check at run time
+  Pandoc->require(1.12);  # check at run time
 
   # execute pandoc
   pandoc 'input.md', -o => 'output.html';
@@ -216,72 +232,37 @@ function C<pandoc> but it can also be used as class.
 
 =head2 pandoc [ @arguments [, \%options ] ]
 
-=head2 pandoc [ \@arguments [, %options ] ]
-
-=head2 pandoc [ \@arguments [, \%options ] ]
-
-Runs the pandoc executable with given command line arguments and options
-and input/output/error redirected, as specified with the in/out/err
-L<options|/"Options">.
-
-Either of C<@arguments> and C<%options>, or both, may be passed as an
-array or hash reference respectively. The items of the argument list to
-C<pandoc()> is interpreted according to these rules:
-
-=over
-
-=item If the first item is an array ref and the last is a hash ref
-
-these are C<\@arguments> and C<\%options> respectively and no other
-items are allowed.
-
-=item If the first item is an array ref and the last is I<not> a hash ref
-
-the first item is C<\@arguments> and the remaining items if any
-(of which there must be an even number) are C<%options>.
-
-This is useful in the common case where the command line arguments are
-the same over multiple calls, while the in/out/err L<options|/"Options">
-are different for each call.
-
-=item If the first item is I<not> an array ref and the last is a hash ref
-
-the last item is C<\%options> and the preceding items if any are
-C<@arguments>.
-
-=item If I<neither> the first item is an array ref I<nor> the last is a hash ref
-
-All the items (if any) are C<@arguments>.
-
-=back
-
-Note that C<\@arguments> must be the first item and C<\%options> must be
-the last, but either may be an empty array/hash reference.
+=head2 pandoc [ \@arguments [, %options | \%options ] ]
 
 If called without arguments and options, the function returns a singleton
-instance of class Pandoc to access information about the executable version of
-pandoc, or C<undef> if no pandoc executable was found.  If called with
-arguments and/or options, the function returns C<0> on success.  Otherwise it
-returns the the exit code of pandoc executable or C<-1> if execution failed.
+instance of class Pandoc to execute L<methods|/METHODS>, or C<undef> if no
+pandoc executable was found. Otherwise runs the pandoc executable with given
+command line arguments. Additional options control input, output, and error
+stream as described below.
+
+Arguments and options can be passed as plain array/hash or as (possibly empty)
+reference but one of them must be a reference if both are provided or if one of
+both is empty.
+
+  pandoc @arguments, { ... };    # ok
+  pandoc [ ... ], %options;      # ok
+
+  pandoc @arguments, %options;   # not ok!
+
+If called with arguments and/or options, the function returns C<0> on success.
+Otherwise it returns the the exit code of pandoc executable or C<-1> if
+execution failed.
 
 =head3 Options
 
 =over
 
-=item in
-
-=item out
-
-=item err
+=item in / out / err
 
 These options correspond to arguments C<$stdin>, C<$stdout>, and
 C<$stderr> of L<IPC::Run3>, see there for details.
 
-=item binmode_stdin
-
-=item binmode_stdout
-
-=item binmode_stderr
+=item binmode_stdin / binmode_stdout / binmode_stderr
 
 These options correspond to the like-named options to L<IPC::Run3>, see
 there for details.
@@ -291,6 +272,10 @@ there for details.
 If defined any binmode_stdin/binmode_stdout/binmode_stderr option which
 is undefined will be set to this value.
 
+=item return_if_system_error
+
+Set to true by default to return the exit code of pandoc executable. 
+
 =back
 
 For convenience the C<pandoc> function (I<after> checking the C<binmode>
@@ -305,40 +290,61 @@ L<use utf8|utf8>, as you then don't need to set the binmode options at
 all (L<encode nor decode|Encode>) when passing input/output scalar
 references.
 
-The C<return_if_system_error> option of L<IPC::Run3> is set to true by default;
-the C<pandoc> function returns the exit code from the pandoc executable.
-
 =head1 METHODS
 
-=head2 new
+=head2 new( [ %options ] )
 
 Create a new instance of class Pandoc or throw an exception if no pandoc
 executable was found. Repeated use of this constructor is not recommended
-unless you explicitly want to call C<pandoc --version>, for instance because
-the system environment has changed during runtime.
+because C<pandoc --version> is called onec for every instance. Possible options
+include:
+
+=head3 Options
+
+=over
+
+=item bin
+
+pandoc executable (C<pandoc> by default)
+
+=back
 
 =head2 run( [ @arguments, \%options ] )
 
-=head2 run( [ \@arguments, %options ] )
-
-=head2 run( [ \@arguments, \%options ] )
+=head2 run( [ \@arguments [ %options | \%options ] ] )
 
 Execute the pandoc executable (see function C<pandoc> above).
 
+=head2 require( $minimum_version )
+
+Return the Pandoc instance if its version number is at least as high as the
+given minimum version. Throw an error otherwise.  This method can also be
+called as constructor: C<< Pandoc->require(...) >> is equivalent to C<<
+pandoc->require >> but throws a more meaningful error message if no pandoc
+executable was found.
+
 =head2 convert( $from => $to, $input [, @arguments ] )
 
 Convert a string in format C<$from> to format C<$to>. Additional pandoc options
-such as C<--smart> and C<--standalone> can be passed. The result is returned 
+such as C<--smart> and C<--standalone> can be passed. The result is returned
 in same utf8 mode (C<utf8::is_unicode>) as the input.
 
-=head2 version( [ $version ] )
+=head2 version( [ $minimum_version ] )
 
-Return the pandoc version if it is at least as new as a given version or if no
-argument was provided.
+Return the pandoc version as L<version> object. Returns undef if the version is
+lower than a given minimum version.
 
-=head2 require( $version )
+=head2 data_dir
 
-Throw an error if the pandoc version is lower than a given version.
+Return the default data directory (only available since Pandoc 1.11).
+
+=head2 input_formats
+
+Return a list of supported input formats.
+
+=head2 output_formats
+
+Return a list of supported output formats.
 
 =head1 SEE ALSO
 
